@@ -14,6 +14,7 @@ namespace Tomboy
 		string backup_dir;
 		ArrayList notes;
 		PluginManager plugin_mgr;
+		TrieController trie_controller;
 
 		public NoteManager () : 
 			this (Path.Combine (Environment.GetEnvironmentVariable ("HOME"), 
@@ -38,15 +39,15 @@ namespace Tomboy
 				Directory.CreateDirectory (notes_dir);
 			}
 
-			// Create & populate the Plugins dir if it doesn't exist
+			// Create & populate the Plugins dir
 			string plugins_dir = Path.Combine (notes_dir, "Plugins");
-			if (!Directory.Exists (plugins_dir))
-				PluginManager.CreatePluginsDir (plugins_dir);
+			PluginManager.CreatePluginsDir (plugins_dir);
 
 			plugin_mgr = new PluginManager (plugins_dir);
+			trie_controller = new TrieController (this);
 
 			if (first_run) {
-				// First run. Create "Start Here" note
+				// First run. Create "Start Here" note.
 				CreateStartNote ();
 			} else {
 				string [] files = Directory.GetFiles (notes_dir, "*.note");
@@ -59,7 +60,10 @@ namespace Tomboy
 					}
 				}
 
-				// Load all the plugins for our notes
+				// Update the trie so plugins can access it, if they want.
+				trie_controller.Update ();
+
+				// Load all the plugins for our notes.
 				foreach (Note note in notes) {
 					plugin_mgr.LoadPluginsForNote (note);
 				}
@@ -84,8 +88,10 @@ namespace Tomboy
 					       "</note-content>",
 					       Catalog.GetString ("Start Here"),
 					       Catalog.GetString ("Welcome to Tomboy!"),
-					       Catalog.GetString ("Use this page as a Start Page for organizing your " +
-								  "notes and keeping unorganized ideas around."));
+					       Catalog.GetString ("Use this page as a Start Page " +
+								  "for organizing your notes and " +
+								  "keeping unorganized ideas " +
+								  "around."));
 
 			Note start_note = Create (Catalog.GetString ("Start Here"), content);
 
@@ -155,22 +161,24 @@ namespace Tomboy
 
 		// Create a new note with the specified title, and a simple
 		// "Describe..." body which will be selected for easy overwrite.
-		public Note Create (string linked_title) 
+		public Note Create (string title) 
 		{
-			string header = linked_title + "\n\n";
+			string header = title + "\n\n";
 			string content = 
 				String.Format ("<note-content>{0}{1}</note-content>",
 					       XmlEncoder.Encode (header),
 					       Catalog.GetString ("Describe your new note here."));
 
-			Note new_note = Create (linked_title, content);
+			Note new_note = Create (title, content);
 
-			// Select the inital "Describe..." text so typing will
-			// immediately overwrite...
-			NoteBuffer buffer = new_note.Buffer;
-			Gtk.TextIter iter = buffer.GetIterAtOffset (header.Length);
-			buffer.MoveMark (buffer.SelectionBound, iter);
-			buffer.MoveMark (buffer.InsertMark, buffer.EndIter);
+			if (new_note != null) {
+			    // Select the inital "Describe..." text so typing will
+			    // immediately overwrite...
+			    NoteBuffer buffer = new_note.Buffer;
+			    Gtk.TextIter iter = buffer.GetIterAtOffset (header.Length);
+			    buffer.MoveMark (buffer.SelectionBound, iter);
+			    buffer.MoveMark (buffer.InsertMark, buffer.EndIter);
+			}
 
 			return new_note;
 		}
@@ -178,9 +186,16 @@ namespace Tomboy
 		// Create a new note with the specified Xml content
 		public Note Create (string title, string xml_content)
 		{
+			if (title == null)
+				return null;
+
+			string trim_title = title.Trim();
+			if (trim_title == string.Empty)
+				return null;
+
 			string filename = MakeNewFileName ();
 
-			Note new_note = new Note (title, filename, this);
+			Note new_note = new Note (trim_title, filename, this);
 			new_note.XmlContent = xml_content;
 			new_note.Renamed += OnNoteRename;
 
@@ -246,8 +261,60 @@ namespace Tomboy
 			}
 		}
 
+		public TrieTree TitleTrie
+		{
+			get {
+				return trie_controller.TitleTrie;
+			}
+		}
+
 		public event NotesChangedHandler NoteDeleted;
 		public event NotesChangedHandler NoteAdded;
 		public event NoteRenameHandler NoteRenamed;
 	}
+
+	class TrieController
+	{
+		TrieTree title_trie;
+		NoteManager manager;
+
+		public TrieController (NoteManager manager)
+		{
+			this.manager = manager;
+			manager.NoteDeleted += OnNoteDeleted;
+			manager.NoteAdded += OnNoteAdded;
+			manager.NoteRenamed += OnNoteRenamed;
+
+			Update ();
+		}
+
+		void OnNoteAdded (object sender, Note added)
+		{
+			Update ();
+		}
+
+		void OnNoteDeleted (object sender, Note deleted)
+		{
+			Update ();
+		}
+
+		void OnNoteRenamed (Note renamed, string old_title)
+		{
+			Update ();
+		}
+
+		public void Update ()
+		{
+			title_trie = new TrieTree (false /* !case_sensitive */);
+
+			foreach (Note note in manager.Notes) {
+				title_trie.AddKeyword (note.Title, note);
+			}
+		}
+
+		public TrieTree TitleTrie 
+		{
+			get { return title_trie; }
+		}
+	}
 }
