@@ -12,8 +12,8 @@ namespace Tomboy
 			: base (buffer)
 		{
 			WrapMode = Gtk.WrapMode.Word;
-			LeftMargin = 8;
-			RightMargin = 8;
+			LeftMargin = DefaultMargin;
+			RightMargin = DefaultMargin;
 			CanDefault = true;
 
 			// Make sure the cursor position is visible
@@ -21,7 +21,8 @@ namespace Tomboy
 
 			// Set Font from GConf preference
 			if ((bool) Preferences.Get (Preferences.ENABLE_CUSTOM_FONT)) {
-				string font_string = (string) Preferences.Get (Preferences.CUSTOM_FONT_FACE);
+				string font_string = (string) 
+					Preferences.Get (Preferences.CUSTOM_FONT_FACE);
 				ModifyFont (Pango.FontDescription.FromString (font_string));
 			}
 			Preferences.SettingChanged += OnFontSettingChanged;
@@ -43,6 +44,11 @@ namespace Tomboy
 		[DllImport("libgtk-win32-2.0-0.dll")]
 		static extern IntPtr gtk_drag_dest_get_target_list (IntPtr raw);
 
+		public static int DefaultMargin
+		{
+			get { return 8; }
+		}
+
 		//
 		// Update the font based on the changed Preference dialog setting.
 		//
@@ -54,7 +60,8 @@ namespace Tomboy
 						   (bool) args.Value ? "ON" : "OFF");
 
 				if ((bool) args.Value) {
-					string font_string = (string) Preferences.Get (Preferences.CUSTOM_FONT_FACE);
+					string font_string = (string) 
+						Preferences.Get (Preferences.CUSTOM_FONT_FACE);
 					ModifyFont (Pango.FontDescription.FromString (font_string));
 				} else
 					ModifyFont (new Pango.FontDescription ());
@@ -109,9 +116,12 @@ namespace Tomboy
 						insert.Append (uri.ToString ());
 				}
 
-				if (insert.Length > 0)
-					Buffer.Insert (Buffer.GetIterAtMark (Buffer.InsertMark),
-						       insert.ToString ());
+				if (insert.Length > 0) {
+					Buffer.InsertWithTags (
+						Buffer.GetIterAtMark (Buffer.InsertMark),
+						insert.ToString (),
+						Buffer.TagTable.Lookup ("link:url"));
+				}
 
 				Gtk.Drag.Finish (context, insert.Length > 0, false, time);
 			} else {
@@ -127,8 +137,8 @@ namespace Tomboy
 		Gtk.AccelGroup accel_group;
 		Gtk.Toolbar toolbar;
 		Gtk.Widget link_button;
-		Gtk.Button text_button;
 		NoteTextMenu text_menu;
+		Gtk.Menu plugin_menu;
 		Gtk.TextView editor;
 		Gtk.ScrolledWindow editor_window;
 
@@ -159,13 +169,12 @@ namespace Tomboy
 			accel_group = new Gtk.AccelGroup ();
 			AddAccelGroup (accel_group);
 
+			text_menu = new NoteTextMenu (accel_group, note.Buffer, note.Buffer.Undoer);
+			plugin_menu = MakePluginMenu ();
+
 			toolbar = MakeToolbar ();
 			toolbar.Show ();
 
-			text_menu = new NoteTextMenu (accel_group, note.Buffer, note.Buffer.Undoer);
-			text_menu.AttachToWidget (text_button, null);
-			text_menu.Deactivated += ReleaseTextButton;
-
 			// The main editor widget
 			editor = new NoteEditor (note.Buffer);
 			editor.PopulatePopup += OnPopulatePopup;
@@ -210,6 +219,12 @@ namespace Tomboy
 						    0,
 						    Gtk.AccelFlags.Visible);
 
+			// Close all windows on current Desktop (Ctrl-Q)
+			global_keys.AddAccelerator (new EventHandler (CloseAllWindowsHandler),
+						    (uint) Gdk.Key.q, 
+						    Gdk.ModifierType.ControlMask,
+						    Gtk.AccelFlags.Visible);
+
 			// Open Find Dialog (Ctrl-F)
 			global_keys.AddAccelerator (new EventHandler (FindActivate),
 						    (uint) Gdk.Key.f, 
@@ -263,6 +278,26 @@ namespace Tomboy
 			Hide ();
 		}
 
+		[DllImport("libtomboy")]
+		static extern int tomboy_window_get_workspace (IntPtr win_raw);
+
+		void CloseAllWindowsHandler (object sender, EventArgs args)
+		{
+			int workspace = tomboy_window_get_workspace (note.Window.Handle);
+
+			foreach (Note iter in note.Manager.Notes) {
+				if (!iter.IsOpened)
+					continue;
+
+				// Close windows on the same workspace, or all
+				// open windows if no workspace.
+				if (workspace < 0 ||
+				    tomboy_window_get_workspace (iter.Window.Handle) == workspace) {
+					iter.Window.CloseWindowHandler (null, null);
+				}
+			}
+		}
+
 		//
 		// Delete this Note.
 		//
@@ -313,6 +348,14 @@ namespace Tomboy
 			get { return toolbar; }
 		}
 
+		public Gtk.Menu PluginMenu {
+			get { return plugin_menu; }
+		}
+
+		public Gtk.Menu TextMenu {
+			get { return text_menu; }
+		}
+
 		// 
 		// Sensitize the Link toolbar button on text selection
 		//
@@ -344,7 +387,8 @@ namespace Tomboy
 			// Remove the lame-o gigantic Insert Unicode Control
 			// Characters menu item.
 			Gtk.Widget lame_unicode;
-			lame_unicode = (Gtk.Widget) args.Menu.Children [args.Menu.Children.Length - 1];
+			lame_unicode = (Gtk.Widget) 
+				args.Menu.Children [args.Menu.Children.Length - 1];
 			args.Menu.Remove (lame_unicode);
 
 			Gtk.MenuItem spacer1 = new Gtk.SeparatorMenuItem ();
@@ -385,8 +429,18 @@ namespace Tomboy
 			args.Menu.Prepend (find_item);
 			args.Menu.Prepend (link);
 
+			Gtk.MenuItem close_all = 
+				new Gtk.MenuItem (Catalog.GetString ("Clos_e All Notes"));
+			close_all.Activated += CloseAllWindowsHandler;
+			close_all.AddAccelerator ("activate",
+						  accel_group,
+						  (uint) Gdk.Key.q, 
+						  Gdk.ModifierType.ControlMask,
+						  Gtk.AccelFlags.Visible);
+			close_all.Show ();
+
 			Gtk.ImageMenuItem close_window = 
-				new Gtk.ImageMenuItem (Catalog.GetString ("_Close Window"));
+				new Gtk.ImageMenuItem (Catalog.GetString ("_Close"));
 			close_window.Image = new Gtk.Image (Gtk.Stock.Close, Gtk.IconSize.Menu);
 			close_window.Activated += CloseWindowHandler;
 			close_window.AddAccelerator ("activate",
@@ -396,19 +450,10 @@ namespace Tomboy
 						     Gtk.AccelFlags.Visible);
 			close_window.Show ();
 
+			args.Menu.Append (close_all);
 			args.Menu.Append (close_window);
 		}
 
-		void UndoClicked (object sender, EventArgs args)
-		{
-			text_menu.UndoClicked (sender, args);
-		}
-
-		void RedoClicked (object sender, EventArgs args)
-		{
-			text_menu.RedoClicked (sender, args);
-		}
-
 		//
 		// Toolbar
 		//
@@ -448,20 +493,27 @@ namespace Tomboy
 						    Gdk.ModifierType.ControlMask,
 						    Gtk.AccelFlags.Visible);
 
-			text_button = new TextToolButton (toolbar);
-			text_button.ButtonPressEvent += TextButtonPress;
-			text_button.Clicked += TextButtonClicked;
+			ToolMenuButton text_button = 
+				new ToolMenuButton (toolbar,
+						    Gtk.Stock.SelectFont,
+						    Catalog.GetString ("_Text"),
+						    text_menu);
+			text_button.IsImportant = true;
 			text_button.Show ();
-
-			// Give it a window to receive events events
-			Gtk.EventBox ev = new Gtk.EventBox ();
-			ev.Add (text_button);
-			ev.Show ();
-
-			toolbar.AppendWidget (ev, 
+			toolbar.AppendWidget (text_button, 
 					      Catalog.GetString ("Set properties of text"), 
 					      null);
 
+			ToolMenuButton plugin_button = 
+				new ToolMenuButton (toolbar, 
+						    Gtk.Stock.Execute,
+						    Catalog.GetString ("_Plugins"),
+						    plugin_menu);
+			plugin_button.Show ();
+			toolbar.AppendWidget (plugin_button, 
+					      Catalog.GetString ("Run Plugin actions"), 
+					      null);
+
 			toolbar.AppendSpace ();
 
 		        Gtk.Widget delete = 
@@ -479,83 +531,44 @@ namespace Tomboy
 			return toolbar;
 		}
 
-		class TextToolButton : Gtk.Button
+		//
+		// Plugin toolbar menu
+		//
+		// Prefixed with Open Plugins Folder action, the rest being
+		// populated by individual plugins using
+		// NotePlugin.AddPluginMenuItem().
+		//
+
+		Gtk.Menu MakePluginMenu ()
 		{
-			Gtk.Image image;
-			Gtk.Label label_horiz;
-			Gtk.Label label_vert;
-			Gtk.Arrow arrow;
-			Gtk.VBox box_vert;
+			Gtk.Menu menu = new Gtk.Menu ();
 
-			public TextToolButton (Gtk.Toolbar toolbar)
-			{
-				this.CanFocus = false;
-				this.Relief = Gtk.ReliefStyle.None;
+			Gtk.ImageMenuItem open;
+			open = new Gtk.ImageMenuItem (Catalog.GetString ("_Open Plugins Folder"));
+			open.Image = new Gtk.Image (Gtk.Stock.Open, Gtk.IconSize.Menu);
+			open.Activated += OnOpenPluginsFolderActivate;
+			open.Show ();
+			menu.Add (open);
 
-				image = new Gtk.Image (Gtk.Stock.SelectFont, toolbar.IconSize);
-				label_horiz = new Gtk.Label (Catalog.GetString ("_Text"));
-				label_vert = new Gtk.Label (Catalog.GetString ("_Text"));
-				arrow = new Gtk.Arrow (Gtk.ArrowType.Down, Gtk.ShadowType.In);
+			Gtk.SeparatorMenuItem sep = new Gtk.SeparatorMenuItem ();
+			sep.Show ();
+			menu.Add (sep);
 
-				box_vert = new Gtk.VBox (false, 0);
-				box_vert.PackStart (image, true, true, 0);
-				box_vert.PackStart (label_vert, false, true, 0);
-
-				Gtk.HBox box_horiz = new Gtk.HBox (false, 0);
-				box_horiz.PackStart (box_vert, true, true, 0);
-				box_horiz.PackStart (label_horiz, true, true, 2);
-				box_horiz.PackStart (arrow, false, true, 0);
-
-				this.Add (box_horiz);
-				this.ShowAll ();
-				ShowForToolbarStyle (toolbar.ToolbarStyle);
-
-				toolbar.StyleChanged += OnToolbarStyleChanged;
-			}
-
-			protected override bool OnButtonPressEvent (Gdk.EventButton ev)
-			{
-				base.OnButtonPressEvent (ev);
-				return false;
-			}
-
-			void ShowForToolbarStyle (Gtk.ToolbarStyle style)
-			{
-				switch (style) {
-				case Gtk.ToolbarStyle.Icons:
-					label_horiz.Hide ();
-					label_vert.Hide ();
-					image.Show ();
-					break;
-				case Gtk.ToolbarStyle.Text:
-					label_vert.Hide ();
-					image.Hide ();
-					label_horiz.Show ();
-					break;
-				case Gtk.ToolbarStyle.Both:
-					label_horiz.Hide ();
-					image.Show ();
-					label_vert.Show ();
-					break;
-				case Gtk.ToolbarStyle.BothHoriz:
-					label_vert.Hide ();
-					image.Show ();
-					label_horiz.Show ();
-					break;
-				}				
-			}
-
-			void OnToolbarStyleChanged (object sender, Gtk.StyleChangedArgs args)
-			{
-				ShowForToolbarStyle (args.Style);
-			}
+			return menu;
 		}
 
+		void OnOpenPluginsFolderActivate (object sender, EventArgs args)
+		{
+			note.Manager.PluginManager.ShowPluginsDirectory ();
+		}
+
+		//
 		// Find context menu
 		//
 		// Find, Find Next, Find Previous menu items.  Next nd previous
 		// are only sensitized when there are search results for this
 		// buffer to iterate.
+		//
 
 		Gtk.Menu MakeFindMenu ()
 		{
@@ -670,34 +683,6 @@ namespace Tomboy
 		{
 			LinkButtonClicked ();
 		}
-
-		//
-		// Popup the Text menu.
-		//
-		// Called when clicking the Font toolbar button.
-		//
-
-		void TextButtonPress (object sender, Gtk.ButtonPressEventArgs args) 
-		{
-			text_menu.RefreshState ();
-			GuiUtils.PopupMenu (text_menu, args.Event);
-		}
-
-		//
-		// Call Release on text_button to reset its state when the menu
-		// closes
-		//
-
-		void ReleaseTextButton (object sender, EventArgs args) 
-		{
-			text_button.Release ();
-		}
-
-		void TextButtonClicked (object sender, EventArgs args) 
-		{
-			text_menu.RefreshState ();
-			GuiUtils.PopupMenu (text_menu, null);
-		}
 	}
 
 	public class NoteTextMenu : Gtk.Menu
@@ -773,7 +758,9 @@ namespace Tomboy
 				undo_manager.UndoChanged += UndoChanged;
 			}
 
-			bold = new Gtk.CheckMenuItem (Catalog.GetString ("<b>_Bold</b>"));
+			bold = new Gtk.CheckMenuItem ("<b>" + 
+						      Catalog.GetString ("_Bold") + 
+						      "</b>");
 			MarkupLabel (bold);
 			bold.Data ["Tag"] = "bold";
 			bold.Activated += FontStyleClicked;
@@ -783,7 +770,9 @@ namespace Tomboy
 					     Gdk.ModifierType.ControlMask,
 					     Gtk.AccelFlags.Visible);
 
-			italic = new Gtk.CheckMenuItem (Catalog.GetString ("<i>_Italic</i>"));
+			italic = new Gtk.CheckMenuItem ("<i>" + 
+							Catalog.GetString ("_Italic") + 
+							"</i>");
 			MarkupLabel (italic);
 			italic.Data ["Tag"] = "italic";
 			italic.Activated += FontStyleClicked;
@@ -793,7 +782,9 @@ namespace Tomboy
 					       Gdk.ModifierType.ControlMask,
 					       Gtk.AccelFlags.Visible);
 
-			strikeout = new Gtk.CheckMenuItem (Catalog.GetString ("<s>_Strikeout</s>"));
+			strikeout = new Gtk.CheckMenuItem ("<s>" + 
+							   Catalog.GetString ("_Strikeout") + 
+							   "</s>");
 			MarkupLabel (strikeout);
 			strikeout.Data ["Tag"] = "strikethrough";
 			strikeout.Activated += FontStyleClicked;
@@ -867,6 +858,12 @@ namespace Tomboy
 			ShowAll ();
 		}
 
+		protected override void OnShown ()
+		{
+			RefreshState ();
+			base.OnShown ();
+		}
+
 		void MarkupLabel (Gtk.MenuItem item)
 		{
 			Gtk.Label label = (Gtk.Label) item.Child;
@@ -913,7 +910,6 @@ namespace Tomboy
 			event_freeze = false;
 		}
 
-
 		// 
 		// Font-style menu item activate
 		//
@@ -958,7 +954,7 @@ namespace Tomboy
 				buffer.SetActiveTag (tag);
 		}
 
-		internal void UndoClicked (object sender, EventArgs args)
+		void UndoClicked (object sender, EventArgs args)
 		{
 			if (undo_manager.CanUndo) {
 				Console.WriteLine ("Running undo...");
@@ -966,7 +962,7 @@ namespace Tomboy
 			}
 		}
 
-		internal void RedoClicked (object sender, EventArgs args)
+		void RedoClicked (object sender, EventArgs args)
 		{
 			if (undo_manager.CanRedo) {
 				Console.WriteLine ("Running redo...");
