@@ -22,6 +22,11 @@ namespace Tomboy
 			// Do nothing.
 		}
 
+		Gtk.TextTag TitleTag
+		{
+			get { return Buffer.TagTable.Lookup ("note-title"); }
+		}
+
 		Gtk.TextIter TitleEnd 
 		{
 			get {
@@ -50,7 +55,7 @@ namespace Tomboy
 
 			// Clean up title line
 			Buffer.RemoveAllTags (TitleStart, TitleEnd);
-			Buffer.ApplyTag ("note-title", TitleStart, TitleEnd);
+			Buffer.ApplyTag (TitleTag, TitleStart, TitleEnd);
 		}
 
 		// This only gets called on an explicit move, not when typing
@@ -68,7 +73,7 @@ namespace Tomboy
 			end.ForwardToLineEnd ();
 
 			// Avoid lingering note-title after a multi-line insert...
-			Buffer.RemoveTag ("note-title", TitleEnd, end);
+			Buffer.RemoveTag (TitleTag, TitleEnd, end);
 		}
 
 		void OnDeleteRange (object sender, Gtk.DeleteRangeArgs args)
@@ -100,7 +105,7 @@ namespace Tomboy
 
 			// Make sure the title line is big and red...
 			Buffer.RemoveAllTags (TitleStart, TitleEnd);
-			Buffer.ApplyTag ("note-title", TitleStart, TitleEnd);
+			Buffer.ApplyTag (TitleTag, TitleStart, TitleEnd);
 
 			// NOTE: Use "(Untitled #)" for empty first lines...
 			string title = TitleStart.GetText (TitleEnd).Trim ();
@@ -178,144 +183,6 @@ namespace Tomboy
 		}
 	}
 
-	public class NoteRelatedToWatcher : NotePlugin
-	{
-		ArrayList linked_from;
-		ArrayList linked_to;
-
-		protected override void Initialize ()
-		{
-			linked_from = new ArrayList ();
-			linked_to = new ArrayList ();
-
-			LinkCreated += OnLinkCreated;
-		}
-
-		protected override void Shutdown ()
-		{
-			// Do nothing.
-		}
-
-		protected override void OnNoteOpened ()
-		{
-			Buffer.TagApplied += OnTagApplied;
-		}
-
-		void OnTagApplied (object sender, Gtk.TagAppliedArgs args)
-		{
-			if (args.Tag.Name != "link:internal") 
-				return;
-
-			string link_text = args.StartChar.GetText (args.EndChar);
-
-			// Ignore self-references
-			if (link_text.ToLower () == Note.Title.ToLower ())
-				return;
-
-			linked_to.Add (link_text);
-
-			Console.WriteLine ("Link Tag applied to '{0}', for '{1}'",
-					   Note.Title,
-					   link_text);
-
-			LinkCreated (Note, link_text);
-		}
-
-		void OnTagRemoved (object sender, Gtk.TagAppliedArgs args)
-		{
-			if (args.Tag.Name != "link:internal")
-				return;
-
-			Console.WriteLine ("Tag 'link:internal' removed!!!");
-		}
-
-		void OnLinkCreated (Note note, string link_text)
-		{
-			if (link_text.ToLower () != Note.Title.ToLower ())
-				return;
-
-			Console.WriteLine ("Received Link text from '{0}' = '{1}'",
-					   note.Title,
-					   link_text);
-
-			// Don't show links we already link to in the body
-			if (linked_to.Contains (link_text))
-				return;
-
-			if (linked_from.Contains (note.Title))
-				return;
-
-			linked_from.Add (note.Title);
-
-			Gtk.TextTag tag = Buffer.TagTable.Lookup ("related-to");
-			Gtk.TextIter iter = Buffer.StartIter;
-
-			if (iter.ForwardToTagToggle (tag)) 
-				RemoveRelatedLine ();
-
-			InsertRelatedLine ();
-		}
-
-		void InsertRelatedLine ()
-		{
-			Gtk.TextIter line, line_end;
-
-			line = Buffer.StartIter;
-			line.ForwardLines (1);
-
-			Buffer.Insert (line, Catalog.GetString ("Related to: "));
-
-			line = Buffer.StartIter;
-			line.ForwardLines (1);
-			line_end = line;
-			line_end.ForwardToLineEnd ();
-
-			Buffer.ApplyTag ("italic", line, line_end);
-
-			line = Buffer.StartIter;
-			line.ForwardLines (1);
-			line_end = line;
-			line_end.ForwardToLineEnd ();
-
-			string link_text = string.Empty;
-			foreach (string link in linked_from) {
-				if (link_text == string.Empty)
-					link_text = link;
-				else {
-					link_text += ", ";
-					link_text += link;
-				}
-			}
-			link_text += "\n";
-
-			Buffer.Insert (line_end, link_text);
-
-			line = Buffer.StartIter;
-			line.ForwardLines (1);
-			line_end = line;
-			line_end.ForwardToLineEnd ();
-
-			Buffer.ApplyTag ("related-to", line, line_end);
-		}
-
-		void RemoveRelatedLine ()
-		{
-			Gtk.TextTag tag = Buffer.TagTable.Lookup ("related-to");
-
-			Gtk.TextIter line = Buffer.StartIter;
-			line.ForwardToTagToggle (tag);
-
-			Gtk.TextIter line_end = line;
-			line_end.ForwardToTagToggle (tag);
-			
-			Buffer.Delete (line, line_end);
-		}
-
-		delegate void LinkCreatedHandler (Note note, string linked_text);
-
-		static event LinkCreatedHandler LinkCreated;
-	}
-
 	public class NoteSpellChecker : NotePlugin
 	{
 		IntPtr obj_ptr = IntPtr.Zero;
@@ -344,10 +211,7 @@ namespace Tomboy
 			Preferences.SettingChanged += OnEnableSpellcheckChanged;
 
 			if ((bool) Preferences.Get (Preferences.ENABLE_SPELLCHECKING)) {
-				obj_ptr = gtkspell_new_attach (Window.Editor.Handle, 
-							       null, 
-							       IntPtr.Zero);
-				FixupOldGtkSpell ();
+				Attach ();
 			}
 		}
 
@@ -376,11 +240,31 @@ namespace Tomboy
 		// use the "error" underline.
 		void FixupOldGtkSpell ()
 		{
+#if OLD_GTKSPELL
 			Gtk.TextTag misspell = Buffer.TagTable.Lookup ("gtkspell-misspelled");
 			if (misspell != null) {
 				OldGtkSpellFixup hack = new OldGtkSpellFixup (misspell.Handle);
 				hack.Fixup ();
 			}
+#endif
+		}
+
+		void Attach ()
+		{
+			if (obj_ptr == IntPtr.Zero) {
+				obj_ptr = gtkspell_new_attach (Window.Editor.Handle, 
+							       null, 
+							       IntPtr.Zero);
+				FixupOldGtkSpell ();
+			}
+		}
+
+		void Detach ()
+		{
+			if (obj_ptr != IntPtr.Zero) {
+				gtkspell_detach (obj_ptr);
+				obj_ptr = IntPtr.Zero;
+			}
 		}
 
 		void OnEnableSpellcheckChanged (object sender, GConf.NotifyEventArgs args)
@@ -389,17 +273,9 @@ namespace Tomboy
 				return;
 
 			if ((bool) args.Value) {
-				if (obj_ptr == IntPtr.Zero) {
-					obj_ptr = gtkspell_new_attach (Window.Editor.Handle, 
-								       null, 
-								       IntPtr.Zero);
-					FixupOldGtkSpell ();
-				}
+				Attach ();
 			} else {
-				if (obj_ptr != IntPtr.Zero) {
-					gtkspell_detach (obj_ptr);
-					obj_ptr = IntPtr.Zero;
-				}
+				Detach ();
 			}
 		}
 
@@ -420,7 +296,8 @@ namespace Tomboy
 			} else if (args.Tag.Name != null && 
 				   (args.Tag.Name.StartsWith ("link:") ||
 				    args.Tag.Name == "note-title")) {
-				Gtk.TextTag misspell = Buffer.TagTable.Lookup ("gtkspell-misspelled");
+				Gtk.TextTag misspell = 
+					Buffer.TagTable.Lookup ("gtkspell-misspelled");
 				if (misspell != null) {
 					Buffer.RemoveTag ("gtkspell-misspelled", 
 							  args.StartChar, 
@@ -433,16 +310,19 @@ namespace Tomboy
 	public class NoteUrlWatcher : NotePlugin
 	{
 		Gtk.TextTag url_tag;
+		Gtk.TextMark click_mark;
 
 		const string URL_REGEX = 
 			@"((\b((news|http|https|ftp|file|irc)://|mailto:|(www|ftp)\.|\S*@\S*\.)|(^|\s)/\S+/|(^|\s)~/\S+)\S*\b/?)";
 
 		static Regex regex;
+		static bool text_event_connected;
 
 		static NoteUrlWatcher ()
 		{
 			regex = new Regex (URL_REGEX, 
 					   RegexOptions.IgnoreCase | RegexOptions.Compiled);
+			text_event_connected = false;
 		}
 
 		protected override void Initialize ()
@@ -463,33 +343,39 @@ namespace Tomboy
 				return;
 			}
 
+#if FIXED_GTKSPELL
+			// NOTE: This hack helps avoid multiple URL opens for
+			// cases where the GtkSpell version is fixed to allow
+			// TagTable sharing.  This is because if the TagTable is
+			// shared, we will connect to the same Tag's event
+			// source each time a note is opened, and get called
+			// multiple times for each button press.  Fixes bug
+			// #305813.
+			if (!text_event_connected) {
+				url_tag.TextEvent += OnTextEvent;
+				text_event_connected = true;
+			}
+#else
 			url_tag.TextEvent += OnTextEvent;
+#endif
+
+			click_mark = Buffer.CreateMark (null, Buffer.StartIter, true);
 
 			Buffer.InsertText += OnInsertText;
 			Buffer.DeleteRange += OnDeleteRange;
+
+			Window.Editor.ButtonPressEvent += OnButtonPress;
+			Window.Editor.PopulatePopup += OnPopulatePopup;
+			Window.Editor.PopupMenu += OnPopupMenu;
 		}
 
-		void OnTextEvent (object sender, Gtk.TextEventArgs args)
+		string GetUrlAtIter (Gtk.TextIter iter)
 		{
-			Gtk.TextTag tag = (Gtk.TextTag) sender;
+			Gtk.TextIter start = iter;
+			start.BackwardToTagToggle (url_tag);
 
-			if (args.Event.Type != Gdk.EventType.ButtonPress)
-				return;
-
-			Gdk.EventButton button_ev = new Gdk.EventButton (args.Event.Handle);
-			if (button_ev.Button != 1 && button_ev.Button != 2)
-				return;
-
-			/* Don't open link if Shift or Control is pressed */
-			if ((int) (button_ev.State & (Gdk.ModifierType.ShiftMask |
-						      Gdk.ModifierType.ControlMask)) != 0)
-				return;
-
-			Gtk.TextIter start = args.Iter;
-			start.BackwardToTagToggle (tag);
-
-			Gtk.TextIter end = args.Iter;
-			end.ForwardToTagToggle (tag);
+			Gtk.TextIter end = iter;
+			end.ForwardToTagToggle (url_tag);
 
 			string url = start.GetText (end);
 
@@ -519,23 +405,14 @@ namespace Tomboy
 				 !url.StartsWith ("mailto:"))
 				url = "mailto:" + url;
 
-			Console.WriteLine ("Opening url '{0}' on click...", url);
+			return url;
+		}
 
-			try {
+		void OpenUrl (string url) 
+		{
+			if (url != string.Empty) {
+				Console.WriteLine ("Opening url '{0}'...", url);
 				Gnome.Url.Show (url);
-
-				// Close note on middle-click
-				if (button_ev.Button == 2) {
-					Window.Hide ();
-
-					// Kill the middle button paste...
-					args.RetVal = true;
-				}
-			} catch (GLib.GException e) {
-				ShowOpeningLocationError (url, e.Message);
-
-				// Kill the middle button paste...
-				args.RetVal = true;
 			}
 		}
 
@@ -554,6 +431,38 @@ namespace Tomboy
 			dialog.Destroy ();
 		}
 
+		void OnTextEvent (object sender, Gtk.TextEventArgs args)
+		{
+			Gtk.TextTag tag = (Gtk.TextTag) sender;
+
+			if (args.Event.Type != Gdk.EventType.ButtonPress)
+				return;
+
+			Gdk.EventButton button_ev = new Gdk.EventButton (args.Event.Handle);
+			if (button_ev.Button != 1 && button_ev.Button != 2)
+				return;
+
+			/* Don't open link if Shift or Control is pressed */
+			if ((int) (button_ev.State & (Gdk.ModifierType.ShiftMask |
+						      Gdk.ModifierType.ControlMask)) != 0)
+				return;
+
+			string url = GetUrlAtIter (args.Iter);
+			try {
+				OpenUrl (url);
+
+				// Close note on middle-click
+				if (button_ev.Button == 2) {
+					Window.Hide ();
+				}
+			} catch (GLib.GException e) {
+				ShowOpeningLocationError (url, e.Message);
+			}
+
+			// Kill the middle button paste...
+			args.RetVal = true;
+		}
+
 		void ApplyUrlToBlock (Gtk.TextIter start, Gtk.TextIter end)
 		{
 			start.LineOffset = 0;
@@ -594,57 +503,80 @@ namespace Tomboy
 
 			ApplyUrlToBlock (start, args.Pos);
 		}
+
+		[GLib.ConnectBefore]
+		void OnButtonPress (object sender, Gtk.ButtonPressEventArgs args)
+		{
+			int x, y;
+
+			Window.Editor.WindowToBufferCoords (Gtk.TextWindowType.Text,
+							    (int) args.Event.X,
+							    (int) args.Event.Y,
+							    out x,
+							    out y);
+			Gtk.TextIter click_iter = Window.Editor.GetIterAtLocation (x, y);
+
+			// Move click_mark to click location
+			Buffer.MoveMark (click_mark, click_iter);
+
+			// Continue event processing
+			args.RetVal = false;
+		}
+
+		void OnPopulatePopup (object sender, Gtk.PopulatePopupArgs args)
+		{
+			Gtk.TextIter click_iter = Buffer.GetIterAtMark (click_mark);
+			if (click_iter.HasTag (url_tag) || click_iter.EndsTag (url_tag)) {
+				Gtk.MenuItem item;
+
+				item = new Gtk.SeparatorMenuItem ();
+				item.Show ();
+				args.Menu.Prepend (item);
+
+				item = new Gtk.MenuItem (Catalog.GetString ("_Copy Link Address"));
+				item.Activated += CopyLinkActivate;
+				item.Show ();
+				args.Menu.Prepend (item);
+
+				item = new Gtk.MenuItem (Catalog.GetString ("_Open Link"));
+				item.Activated += OpenLinkActivate;
+				item.Show ();
+				args.Menu.Prepend (item);
+			}
+		}
+
+		// Called via Alt-F10.  Reset click_mark to cursor location.
+		[GLib.ConnectBefore]
+		void OnPopupMenu (object sender, Gtk.PopupMenuArgs args)
+		{
+			Gtk.TextIter click_iter = Buffer.GetIterAtMark (Buffer.InsertMark);
+			Buffer.MoveMark (click_mark, click_iter);
+			args.RetVal = false; // Continue event processing
+		}
+
+		void OpenLinkActivate (object sender, EventArgs args)
+		{
+			Gtk.TextIter click_iter = Buffer.GetIterAtMark (click_mark);
+			string url = GetUrlAtIter (click_iter);
+			try {
+				OpenUrl (url);
+			} catch (GLib.GException e) {
+				ShowOpeningLocationError (url, e.Message);
+			}
+		}
+
+		void CopyLinkActivate (object sender, EventArgs args)
+		{
+			Gtk.TextIter click_iter = Buffer.GetIterAtMark (click_mark);
+			string url = GetUrlAtIter (click_iter);
+
+			Gtk.Clipboard clip = Window.Editor.GetClipboard (Gdk.Selection.Clipboard);
+			clip.SetText (url);
+		}
 	}
 
 	public class NoteLinkWatcher : NotePlugin
 	{
-		class TrieController
-		{
-			TrieTree title_trie;
-			NoteManager manager;
-
-			public TrieController (NoteManager manager)
-			{
-				this.manager = manager;
-				manager.NoteDeleted += OnNoteDeleted;
-				manager.NoteAdded += OnNoteAdded;
-				manager.NoteRenamed += OnNoteRenamed;
-
-				Update ();
-			}
-
-			void OnNoteAdded (object sender, Note added)
-			{
-				Update ();
-			}
-
-			void OnNoteDeleted (object sender, Note deleted)
-			{
-				Update ();
-			}
-
-			void OnNoteRenamed (Note renamed, string old_title)
-			{
-				Update ();
-			}
-
-			public void Update ()
-			{
-				title_trie = new TrieTree (false /* !case_sensitive */);
-
-				foreach (Note note in manager.Notes) {
-					title_trie.AddKeyword (note.Title, note);
-				}
-			}
-
-			public TrieTree TitleTrie 
-			{
-				get { return title_trie; }
-			}
-		}
-
-		static TrieController trie_controller;
-
 		protected override void Initialize () 
 		{
 			Manager.NoteDeleted += OnNoteDeleted;
@@ -681,32 +613,8 @@ namespace Tomboy
 			if (!ContainsText (added.Title))
 				return;
 
-			Gtk.TextTag url_tag = Buffer.TagTable.Lookup ("link:url");
-			Gtk.TextTag link_tag = Buffer.TagTable.Lookup ("link:internal");
-			Gtk.TextTag broken_link_tag = Buffer.TagTable.Lookup ("link:broken");
-
-			string new_title_lower = added.Title.ToLower ();
-
-			string buffer_text = Buffer.StartIter.GetText (Buffer.EndIter);
-			buffer_text = buffer_text.ToLower ();
-
-			int idx = 0;
-
-			while (true) {
-				idx = buffer_text.IndexOf (new_title_lower, idx);
-				if (idx < 0)
-					break;
-
-				Gtk.TextIter start = Buffer.GetIterAtOffset (idx);
-				Gtk.TextIter end = Buffer.GetIterAtOffset (idx + added.Title.Length);
-
-				if (!start.HasTag (url_tag)) {
-					Buffer.RemoveTag (broken_link_tag, start, end);
-					Buffer.ApplyTag (link_tag, start, end);
-				}
-
-				idx += new_title_lower.Length;
-			}
+			// Highlight previously unlinked text
+			Highlighter.HighlightNote (Note, Buffer.StartIter, Buffer.EndIter, added);
 		}
 
 		void OnNoteDeleted (object sender, Note deleted)
@@ -722,6 +630,7 @@ namespace Tomboy
 
 			string old_title_lower = deleted.Title.ToLower ();
 
+			// Turn all link:internal to link:broken for the deleted note.
 			TextTagEnumerator enumerator = new TextTagEnumerator (Buffer, link_tag);
 			foreach (TextRange range in enumerator) {
 				if (range.Text.ToLower () != old_title_lower)
@@ -734,21 +643,32 @@ namespace Tomboy
 
 		void OnNoteRenamed (Note renamed, string old_title)
 		{
+			if (renamed == this.Note)
+				return;
+
+			// Highlight previously unlinked text
+			if (ContainsText (renamed.Title)) {
+				Highlighter.HighlightNote (Note, 
+							   Buffer.StartIter, 
+							   Buffer.EndIter, 
+							   renamed);
+			}
+
 			if (!ContainsText (old_title))
 				return;
 
 			Gtk.TextTag link_tag = Buffer.TagTable.Lookup ("link:internal");
-
 			string old_title_lower = old_title.ToLower ();
 
+			// Replace existing links with the new title.
 			TextTagEnumerator enumerator = new TextTagEnumerator (Buffer, link_tag);
 			foreach (TextRange range in enumerator) {
-				Console.WriteLine ("RENAME: Checking '{0}' in note '{1}'", range.Text, Note.Title);
-
 				if (range.Text.ToLower () != old_title_lower)
 					continue;
 
-				Console.WriteLine ("Replacing with '{0}'", renamed.Title);
+				Console.WriteLine ("Replacing '{0}' with '{0}'", 
+						   range.Text, 
+						   renamed.Title);
 
 				Buffer.Delete (range.Start, range.End);
 				Buffer.InsertWithTags (range.Start, renamed.Title, link_tag);
@@ -765,9 +685,9 @@ namespace Tomboy
 			Gtk.TextTag link_tag;
 			Gtk.TextTag broken_link_tag;
 
-			public Highlighter (Note         note, 
-					    Gtk.TextIter start, 
-					    Gtk.TextIter end)
+			Highlighter (Note         note, 
+				     Gtk.TextIter start, 
+				     Gtk.TextIter end)
 			{
 				this.note = note;
 				this.start = start;
@@ -778,10 +698,35 @@ namespace Tomboy
 				this.broken_link_tag = note.Buffer.TagTable.Lookup ("link:broken");
 			}
 
-			public void Highlight (TrieTree trie)
+			public static void HighlightTrie (Note         note, 
+							  Gtk.TextIter start, 
+							  Gtk.TextIter end,
+							  TrieTree     trie)
 			{
+				Highlighter high = new Highlighter (note, start, end);
 				trie.FindMatches (start.GetText (end), 
-						  new MatchHandler (TitleFound));
+						  new MatchHandler (high.TitleFound));
+			}
+
+			public static void HighlightNote (Note         note, 
+							  Gtk.TextIter start, 
+							  Gtk.TextIter end,
+							  Note         find_note)
+			{
+				Highlighter high = new Highlighter (note, start, end);
+
+				string buffer_text = start.GetText (end).ToLower();
+				string find_title_lower = find_note.Title.ToLower ();
+				int idx = 0;
+
+				while (true) {
+					idx = buffer_text.IndexOf (find_title_lower, idx);
+					if (idx < 0)
+						break;
+
+					high.TitleFound (buffer_text, idx, find_note);
+					idx += find_title_lower.Length;
+				}
 			}
 
 			void TitleFound (string haystack, int start_idx, object value)
@@ -801,31 +746,18 @@ namespace Tomboy
 				Gtk.TextIter title_end = title_start;
 				title_end.ForwardChars (match_note.Title.Length);
 
-				Console.WriteLine ("Matching Note title '{0}'...", match_note.Title);
+				Console.WriteLine ("Matching Note title '{0}'...", 
+						   match_note.Title);
 
-				note.Buffer.RemoveTag (broken_link_tag,
-						       title_start,
-						       title_end);
-				note.Buffer.ApplyTag (link_tag,
-						      title_start,
-						      title_end);
-			}
-		}
-
-		TrieTree TitleTrie
-		{
-			get {
-				if (trie_controller == null)
-					trie_controller = new TrieController (Manager);
-
-				return trie_controller.TitleTrie;
+				note.Buffer.RemoveTag (broken_link_tag, title_start, title_end);
+				note.Buffer.ApplyTag (link_tag, title_start, title_end);
 			}
 		}
 
 		void HighlightInBlock (Gtk.TextIter start, Gtk.TextIter end) 
 		{
-			Highlighter high = new Highlighter (Note, start, end);
-			high.Highlight (TitleTrie);
+			GetBlockExtents (ref start, ref end);
+			Highlighter.HighlightTrie (Note, start, end, Manager.TitleTrie);
 		}
 
 		void UnhighlightInBlock (Gtk.TextIter start, Gtk.TextIter end) 
@@ -848,8 +780,6 @@ namespace Tomboy
 			Gtk.TextIter start = args.Start;
 			Gtk.TextIter end = args.End;
 
-			GetBlockExtents (ref start, ref end);
-
 			UnhighlightInBlock (start, end);
 			HighlightInBlock (start, end);
 		}
@@ -861,8 +791,6 @@ namespace Tomboy
 
 			Gtk.TextIter end = args.Pos;
 
-			GetBlockExtents (ref start, ref end);
-
 			UnhighlightInBlock (start, end);
 			HighlightInBlock (start, end);
 		}
@@ -896,7 +824,7 @@ namespace Tomboy
 		{
 			broken_link_tag = Buffer.TagTable.Lookup ("link:broken");
 			if (broken_link_tag == null) {
-				Console.WriteLine ("ERROR: Broken link tags not registered for buffer.");
+				Console.WriteLine ("ERROR: Broken link tags not registered.");
 				return;
 			}
 
@@ -989,11 +917,13 @@ namespace Tomboy
 
 		static Gdk.Cursor normal_cursor;
 		static Gdk.Cursor hand_cursor;
+		static bool text_event_connected;
 
 		static MouseHandWatcher ()
 		{
 			normal_cursor = new Gdk.Cursor (Gdk.CursorType.Xterm);
 			hand_cursor = new Gdk.Cursor (Gdk.CursorType.Hand2);
+			text_event_connected = false;
 		}
 
 		protected override void Initialize ()
@@ -1009,10 +939,24 @@ namespace Tomboy
 		protected override void OnNoteOpened ()
 		{
 			link_tag = Buffer.TagTable.Lookup ("link:internal");
-			link_tag.TextEvent += OnLinkTextEvent;
-
 			broken_link_tag = Buffer.TagTable.Lookup ("link:broken");
+
+#if FIXED_GTKSPELL
+			// NOTE: This avoid multiple link opens for cases where
+			// the GtkSpell version is fixed to allow TagTable
+			// sharing.  This is because if the TagTable is shared,
+			// we will connect to the same Tag's event source each
+			// time a note is opened, and get called multiple times
+			// for each button press.  Fixes bug #305813.
+			if (!text_event_connected) {
+				link_tag.TextEvent += OnLinkTextEvent;
+				broken_link_tag.TextEvent += OnLinkTextEvent;
+				text_event_connected = true;
+			}
+#else
+			link_tag.TextEvent += OnLinkTextEvent;
 			broken_link_tag.TextEvent += OnLinkTextEvent;
+#endif
 
 			Gtk.TextView editor = Window.Editor;
 			editor.MotionNotifyEvent += OnEditorMotion;
@@ -1184,325 +1128,4 @@ namespace Tomboy
 			}
 		}
 	}
-
-	public class IndentWatcher : NotePlugin
-	{
-		protected override void Initialize ()
-		{
-			// Do nothing.
-		}
-
-		protected override void Shutdown ()
-		{
-			// Do nothing.
-		}
-
-		protected override void OnNoteOpened ()
-		{
-			Window.Editor.KeyPressEvent += OnEditorKeyPress;
-			Buffer.InsertText += OnInsertText;
-		}
-
-		[GLib.ConnectBefore]
-		void OnEditorKeyPress (object sender, Gtk.KeyPressEventArgs args) 
-		{
-			switch (args.Event.Key) {
-				/*
-			case Gdk.Key.asterisk:
-				Console.WriteLine ("Got Asterisk!");
-
-				{
-				Gtk.TextIter insert = Buffer.GetIterAtMark (Buffer.InsertMark);
-				if (!insert.StartsLine ())
-					break;
-
-				Gtk.TextIter para_end = insert;
-				para_end.ForwardToLineEnd ();
-
-				Console.WriteLine ("Setting hanging indent!");
-
-				Gtk.TextTag indent_tag = new Gtk.TextTag (null);
-				indent_tag.Indent = -10;
-				indent_tag.Data ["has_indent"] = true;
-
-				Buffer.TagTable.Add (indent_tag);
-				Buffer.ApplyTag (indent_tag, insert, para_end);
-				}
-				break;
-				*/
-				
-			case Gdk.Key.Tab:
-			case Gdk.Key.KP_Tab:
-				Console.WriteLine ("Got Tab!");
-
-				Gtk.TextIter insert = Buffer.GetIterAtMark (Buffer.InsertMark);
-				if (!insert.StartsLine ())
-					break;
-
-				Console.WriteLine ("Going to indent!");
-
-				Gtk.TextIter para_end = insert;
-				para_end.ForwardToLineEnd ();
-
-				bool indent_exists = false;
-
-				foreach (Gtk.TextTag tag in insert.Tags) {
-					if (tag.Data ["has_margin"] != null) {
-						Console.WriteLine ("Has Indent Already! Updating!");
-						tag.LeftMargin = tag.LeftMargin + 40;
-						indent_exists = true;
-						break;
-					}
-				}
-
-				if (!indent_exists) {
-					Gtk.TextTag indent_tag = new Gtk.TextTag (null);
-					indent_tag.LeftMargin = 40;
-					indent_tag.Data ["has_margin"] = true;
-
-					Buffer.TagTable.Add (indent_tag);
-					Buffer.ApplyTag (indent_tag, insert, para_end);
-				}
-
-				args.RetVal = true;
-				break;
-
-			case Gdk.Key.BackSpace:
-				Console.WriteLine ("Got Backspace!");
-
-				Gtk.TextIter insert2 = Buffer.GetIterAtMark (Buffer.InsertMark);
-				if (!insert2.StartsLine ())
-					break;
-
-				Console.WriteLine ("Going to unindent!");
-
-				Gtk.TextIter para_end2 = insert2;
-				para_end2.ForwardToLineEnd ();
-
-				foreach (Gtk.TextTag tag in insert2.Tags) {
-					if (tag.Data ["has_margin"] != null) {
-						Console.WriteLine ("Has Indent Already! Updating!");
-						tag.LeftMargin = tag.LeftMargin - 40;
-						if (tag.LeftMargin == 0) {
-							Console.WriteLine ("Killing empty indent!");
-							Buffer.RemoveTag (tag, insert2, para_end2);
-						}
-
-						args.RetVal = true;
-						break;
-					}
-				}
-				break;
-			}
-		}
-
-		void OnInsertText (object sender, Gtk.InsertTextArgs args)
-		{
-			Console.WriteLine ("Trying hanging indent! line-offset:{0}", args.Pos.LineOffset);
-
-			if (args.Text != "*" || args.Pos.LineOffset != 1)
-				return;
-
-			Gtk.TextIter para_start = args.Pos;
-			para_start.BackwardChar ();
-
-			Gtk.TextTag indent_tag = new Gtk.TextTag (null);
-			indent_tag.Indent = -10;
-			indent_tag.Data ["has_indent"] = true;
-
-			Buffer.TagTable.Add (indent_tag);
-			Buffer.ApplyTag (indent_tag, para_start, args.Pos);
-		}
-
-		// 
-		// <list>
-		// <anonymous list-prefix="124">Content content content</anonymous>
-		// <anonymous indent="40">
-		//   <anonymous list-prefix="123">
-		//     Content content content
-		//   </anonymous>
-		// </anonymous>
-		// </list>
-
-		string GetListPrefix (Gtk.TextIter iter)
-		{
-			Gtk.TextTag number_list = Buffer.TagTable.Lookup ("number-list");
-			Gtk.TextTag bullet_list = Buffer.TagTable.Lookup ("bullet-list");
-
-			if (iter.HasTag (bullet_list))
-				return "* ";
-
-			int add = 0;
-
-			while (iter.HasTag (number_list)) {
-				foreach (Gtk.TextTag tag in iter.Tags) {
-					if (tag.Data ["list-prefix"] != null) {
-						int last_num = (int) tag.Data ["list-prefix"];
-						last_num += add;
-						return last_num.ToString () + ". ";
-					}
-				}
-
-				iter.BackwardLine ();
-				add++;
-			}
-
-			return null;
-		}
-	}
-
-#if BROKEN
-	public class SpacingWatcher 
-	{
-		Note note;
-		NoteBuffer buffer;
-		Gtk.TextView editor;
-
-		public SpacingWatcher (Note note)
-		{
-			this.note = note;
-			this.buffer = note.Buffer;
-			this.editor = note.Window.Editor;
-
-			buffer.InsertText += OnInsertText;
-			//buffer.DeleteRange += OnDeleteRange;
-		}
-
-		void OnDeleteRange (object sender, Gtk.DeleteRangeArgs args)
-		{
-		}
-
-		class AddHRule
-		{
-			Gtk.TextBuffer buffer;
-			Gtk.TextMark mark;
-			Gtk.TextView view;
-
-			public AddHRule (Gtk.TextIter start,
-					 Gtk.TextIter end,
-					 Gtk.TextView view)
-			{
-				buffer = view.Buffer;
-				this.view = view;
-				mark = buffer.CreateMark (null, start, true);
-
-				buffer.ApplyTag ("centered", start, end);
-
-				// add this in idle so we don't invalidate text iters
-				GLib.Idle.Add (new GLib.IdleHandler (AddHRuleIdle));
-			}
-
-			bool AddHRuleIdle ()
-			{
-				Console.WriteLine ("Got Separator Line");
-
-				Gtk.TextIter start = buffer.GetIterAtMark (mark);
-
-				Gtk.TextChildAnchor anchor;
-				anchor = buffer.CreateChildAnchor (start);
-
-				Gtk.Widget rule = new Gtk.HSeparator ();
-				rule.WidthRequest = 200;
-				rule.Show ();
-
-				view.AddChildAtAnchor (rule, anchor);
-
-				return false;
-			}
-		}
-
-		void OnInsertText (object sender, Gtk.InsertTextArgs args)
-		{
-			Gtk.TextIter line_start = args.Pos;
-			line_start.LineOffset = 0;
-
-			Gtk.TextIter line_end = args.Pos;
-			line_end.ForwardToLineEnd ();
-
-			if (line_start.GetText (line_end) == "...") {
-				new AddHRule (line_start, line_end, editor);
-			}
-		}
-	}
-
-	public class NoteListWatcher
-	{
-		Note note;
-		NoteBuffer buffer;
-		Gtk.TextView editor;
-
-		// Apply list:bullet, with the number of whitespace characters
-		// at the start of line as the indent level.  On newline, check
-		// the offset of the last line, and copy it (will need to insert
-		// the offset characters in an idle, i guess, or maybe create
-		// and apply a tag on the fly with LeftMargin attribute).  When
-		// unserializing, insert an image/marker at the beginning of
-		// text (need to worry about varing width numbers).
-		//
-		// Can we alter the selection to not select these regions?  This
-		// might be fixed by using an applied tag.
-
-		public NoteListWatcher (Note note)
-		{
-			this.note = note;
-			this.buffer = note.Buffer;
-			this.editor = note.Window.Editor;
-
-			buffer.InsertText += OnInsertText;
-			buffer.DeleteRange += OnDeleteRange;
-		}
-
-		void OnInsertText (object sender, Gtk.InsertTextArgs args)
-		{
-			if (args.Text != "\t" && args.Text != "*")
-				return;
-
-			Gtk.TextIter line_start = args.Pos;
-			line_start.LineOffset = 0;
-
-			if (line_start.Char == string.Empty)
-				return;
-
-			int indent = 0;
-			while (line_start.Char == "\t") {
-				indent++;
-				line_start.ForwardChar ();
-				Console.WriteLine ("Got indent {0}!", indent);
-			}
-
-			if (indent == 0)
-				return;
-
-			if (!line_start.Equal (args.Pos))
-				return;
-
-			foreach (Gtk.TextTag tag in line_start.Tags) {
-				if (tag.Data ["mybutt"] != null) {
-					tag.LeftMargin += 20;
-					break;
-				}
-			}
-
-			Gtk.TextIter line_end = args.Pos;
-			line_end.ForwardToLineEnd ();
-
-			/*
-			if (line_start.Char [0] == '*')
-				new AddBullet (line_start, line_end, editor);
-			else if (line_start.Char [0] == '-')
-				new AddDash (line_start, line_end, editor);
-			else if (Char.IsDigit (line_start.Char [0]))
-				new AddNumber (line_start, line_end, editor);
-			*/
-		}
-
-		[GLib.ConnectBefore]
-		void OnDeleteRange (object sender, Gtk.DeleteRangeArgs args)
-		{
-			if (args.Start.Char == "*")
-				Console.WriteLine ("Unindenting!");
-		}
-	}
-#endif // BROKEN
-
 }
