@@ -248,7 +248,10 @@ namespace Tomboy
 
 		void KeyShowMenu (object sender, EventArgs args)
 		{
-			tray.ShowMenu ();
+			// Show the notes menu, highlighting the first item.
+			// This matches the behavior of GTK for
+			// accelerator-shown menus.
+			tray.ShowMenu (true);
 		}
 
 		void KeyOpenStartHere (object sender, EventArgs args)
@@ -261,7 +264,8 @@ namespace Tomboy
 		void KeyCreateNewNote (object sender, EventArgs args)
 		{
 			Note new_note = manager.Create ();
-			new_note.Window.Show ();
+			if (new_note != null)
+			    new_note.Window.Show ();
 		}
 
 		void KeyOpenSearch (object sender, EventArgs args)
